
--- 


--
-- Example of Pinch To Zoom for Gideros Studio
--

application:setLogicalDimensions(768, 1024)
application:setOrientation(Application.LANDSCAPE_LEFT)


local minimumZoom = 0.2

local function touchEVENT(self, event)
	local touchGet = event.touch
	local id = touchGet.id
	local zoomGroup = self.zoomGroup
	-- id is the finger number
	
	local touch = self.touch
	
	-- It just happen that i like to use some checking like this
	local scalingHappen = false
	local movingHappen = false
	
	local otherID = 2
	if id == 2 then otherID = 1 end
	--Function to get range between touch/coord
	local function getDelta(point1,point2)
		local dx = point1.x - point2.x
		local dy = point1.y - point2.y
		return math.sqrt( dx*dx + dy*dy )
	end
	
	if id == 1 or id == 2 then
		
		if event.type == "touchesBegin" then
			
			if not self.idTouchExist[otherID] then
				
				self.x0 = touchGet.x - ( self:getX()* zoomGroup:getScaleX() )
				self.y0 = touchGet.y - ( self:getY()* zoomGroup:getScaleX() )
				
				--Get the first Finger
				touch[id].x = touchGet.x
				touch[id].y = touchGet.y
				
			elseif not self.idTouchExist[id] then
			
				--Get the second Finger
				touch[id].x = touchGet.x
				touch[id].y = touchGet.y
				
				-- get Length between touch and original Zoom
				local deltaTouch = getDelta(touch[1],touch[2]) 
				if deltaTouch > 0 then
					zoomGroup.deltaTouch = deltaTouch
					zoomGroup.oriScale = zoomGroup:getScaleX()
				end
				
				-- TouchC is a Temporary that i use to take center of 1 & 2
				local touchC = {}
				touchC.x = (touch[1].x + touch[2].x) * 0.5
				touchC.y = (touch[1].y + touch[2].y) * 0.5
				self.x0 = touchC.x - ( self:getX() * zoomGroup:getScaleX() )
				self.y0 = touchC.y - ( self:getY() * zoomGroup:getScaleX() )
				
				
			end
			
			self.idTouchExist[id] = true
			
			
		elseif event.type == "touchesMove" then
			-- save Touch Pos
			touch[id].x = touchGet.x
			touch[id].y = touchGet.y
			
			-- If there is one Touch then move
			if not self.idTouchExist[otherID] then
			
				self:setX( (touchGet.x  - self.x0) * zoomGroup.scaleRev )
				self:setY( (touchGet.y  - self.y0) * zoomGroup.scaleRev )
				movingHappen = true
				
			-- If there is second finter Then Scale (and move)	
			else
			
				local deltaTouch = getDelta(touch[1],touch[2]) 
				local scaler = deltaTouch / zoomGroup.deltaTouch
				
				if scaler > 0 then
					local scaleResult = zoomGroup.oriScale * scaler
					zoomGroup:setScale( scaleResult,scaleResult )
					scalingHappen = true
				end
				
				local touchC = {}
				touchC.x = (touch[1].x + touch[2].x) * 0.5
				touchC.y = (touch[1].y + touch[2].y) * 0.5
				self:setX( (touchC.x  - self.x0 ) * zoomGroup.scaleRev )
				self:setY( (touchC.y  - self.y0 ) * zoomGroup.scaleRev )
			end
			
			event:stopPropagation()

			-- you can put limitation of X,Y max and min in here
			if self:getY() < self.downLimit then self:setY( self.downLimit )
			elseif self:getY() > self.upLimit then self:setY(  self.upLimit )
			end
			if self:getX() < self.rightLimit then self:setX( self.rightLimit )
			elseif self:getX() > self.leftLimit then self:setX(  self.leftLimit )
			end
				
			-- and this for limit scaling
			if scalingHappen then
				if zoomGroup:getScaleX() > 1.0  then
					zoomGroup:setScale( 1.0,1.0 )
				elseif zoomGroup:getScaleX() < minimumZoom then
					zoomGroup:setScale( minimumZoom,minimumZoom )
				end	
				zoomGroup.scaleRev = 1/zoomGroup:getScaleX()
			end
			
			-- Using animation also nice...
			-- So just give some condition to let  ENTER_FRAME event do the scaling / moving after
			-- And then let them move like this:
				-- if self.targetX ~= self:getX() then 
				-- 	local delta = ( self:getX() - self.TargetX ) * 0.1
				--		if delta > 0 then delta = ceil(delta) else delta = floor(delta) end
				--		self:setX( self:getX() - delta )
				-- end 
			-- But it more complexer than that, i think
			
			
		elseif event.type == "touchesEnd" then
		
			if self.idTouchExist[otherID] then
				self.x0 = touch[otherID].x  -( self:getX()* zoomGroup:getScaleX() )
				self.y0 = touch[otherID].y  - ( self:getY()* zoomGroup:getScaleX() )
			end	
			
			event:stopPropagation()
			self.idTouchExist[id] = nil
		end
	end
	
	return true
	
end


local zoomGroup = Sprite.new()
zoomGroup:setScale(0.4,0.4)
zoomGroup:setPosition( application:getContentWidth()/2,application:getContentHeight()/2)

local imageOrGroup = Sprite.new()
imageOrGroup.bitmap = Bitmap.new(Texture.new("Bunny.jpg"))
imageOrGroup.bitmap:setAnchorPoint(0.5, 0.5)
imageOrGroup.zoomGroup = zoomGroup

imageOrGroup:addChild(imageOrGroup.bitmap)

zoomGroup:addChild(imageOrGroup)
stage:addChild(zoomGroup)

-- Some of initialization that i need for Pinch-Zoom
zoomGroup.scaleRev = 1/zoomGroup:getScaleX()

imageOrGroup.touch = {}
imageOrGroup.touch[1] = {}
imageOrGroup.touch[2] = {}
imageOrGroup.idTouchExist = {}


local halfWidth = imageOrGroup:getWidth()*0.5
local halfHeight = imageOrGroup:getHeight()*0.5
imageOrGroup.leftLimit = halfWidth
imageOrGroup.rightLimit = -halfWidth
imageOrGroup.upLimit = halfHeight
imageOrGroup.downLimit = -halfHeight

imageOrGroup:addEventListener(Event.TOUCHES_BEGIN, touchEVENT,imageOrGroup)
imageOrGroup:addEventListener(Event.TOUCHES_MOVE, touchEVENT,imageOrGroup)
imageOrGroup:addEventListener(Event.TOUCHES_END, touchEVENT,imageOrGroup)

